#! /usr/bin/env python
#
# Author: Jean-Marc Legrand
# Date:   Tue May 25 2010
#
# Copyright (C) 2010 Paraxip Technologies
# All Rights Reserved.
#
# This code is Paraxip Technologies Confidential Property.
# Use of and Access to this code is covered by a previously executed
# Non-disclosure Agreement between Paraxip Technologies and the Recipient.
# This code is being supplied for evaluation purposes only and is not to be
# used for any other purpose.

import sys
import datetime
import time

# Imports for SQLalachemy
from sqlalchemy import Integer, String, DateTime, Boolean
from sqlalchemy import Column, ForeignKey
from sqlalchemy import and_, or_
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relation, backref
from sqlalchemy.orm import sessionmaker

# Call data
from paraxip.thread.CallData import *

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

# Init logging
from paraxip.infra.PyLoggingFunctions import *
from paraxip.infra.Assert import *
fileScopeLogger = Logger("netborder.GWCallData")

# GWCallDataRegistry
class GWCallDataRegistry(CallDataRegistry):
    """GWCallDataRegistry"""

    m_blockedCallsQuery = None
    m_busyBlockedCallsQuery = None
    m_noAnswerBlockedCallsQuery = None
    m_notFoundBlockedCallsQuery = None
    
    def __init__(self):        
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.__init__")
        self.m_blockedCallsQuery = None
        self.m_busyBlockedCallsQuery = None
        self.m_noAnswerBlockedCallsQuery = None
        self.m_notFoundBlockedCallsQuery = None
        CallDataRegistry.__init__(self)

    def getDateTimeNow(self):
        d = datetime.datetime.now()
        return str(d)

    def getPstnInCallsFromQuery(self, in_query ):
        return self.getSameDirectionDataFromQuery( in_query, "PSTN IN->" )
    
    def getPstnOutCallsFromQuery(self, in_query ):
        return self.getSameDirectionDataFromQuery( in_query, "->PSTN OUT" )
    
    def getActivePstnInCallsQuery(self, in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getActivePstnInCallsQuery")    
        session = self.m_session if in_session is None else in_session        
        return self.getPstnInCallsFromQuery( self. \
                                             getActiveCallsQuery( session ))

    def getActivePstnInCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getActivePstnInCalls")
        return self.getActivePstnInCallsQuery(in_session).all()

    def getNumActivePstnInCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getNumActivePstnInCalls")
        return self.getActivePstnInCallsQuery(in_session).count()


    def getActivePstnOutCallsQuery(self, in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getActivePstnInCallsQuery")    
        return self.getPstnOutCallsFromQuery(self.\
                                             getActiveCallsQuery(in_session))

    def getActivePstnOutCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getActivePstnOutCalls")
        return self.getActivePstnOutCallsQuery(in_session).all()

    def getNumActivePstnOutCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getNumActivePstnOutCalls")
        return self.getActivePstnOutCallsQuery(in_session).count()

    def getBlockedCallsQuery(self, in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getBlockedCallsQuery")
        session = self.m_session if in_session is None else in_session
        return session.query(CallData).filter(CallData.m_state == "BLOCKED")

    def getBlockedCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getBlockedCalls")
        return self.getBlockedCallsQuery(in_session).all()

    def getNumBlockedCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getNumBlockedCalls")
        return self.getBlockedCallsQuery(in_session).count()

    def getBlockedPstnOutCallsQuery(self, in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getBlockedPstnInCallsQuery")
        return self.getPstnOutCallsFromQuery( self. \
                                              getBlockedCallsQuery(in_session))

    def getBlockedPstnOutCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getBlockedPstnOutCalls")
        return self.getBlockedPstnOutCallsQuery(in_session).all()

    def getNumBlockedPstnOutCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getNumBlockedPstnOutCalls")
        return self.getBlockedPstnOutCallsQuery(in_session).count()

    def getBlockedPstnInCallsQuery(self, in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getBlockedPstnInCallsQuery")    
        return self.getPstnInCallsFromQuery( self. \
                                             getBlockedCallsQuery(in_session))

    def getBlockedPstnInCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getBlockedPstnInCalls")
        return self.getBlockedPstnInCallsQuery(in_session).all()

    def getNumBlockedPstnInCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getNumBlockedPstnInCalls")
        return self.getBlockedPstnInCallsQuery(in_session).count()

    def getConnectedCallsQuery(self, in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger,
                       "GWCallDataRegistry.getConnectedCallsQuery")    
        session = self.m_session if in_session is None else in_session

        # Get all events that leads us to CONNECTED state
        callDataIds = session.query(CallDataEvent.m_callDataId). \
                      filter( CallDataEvent.m_newState == "CONNECTED")

        return session.query(CallData). \
               filter( CallData.m_id.in_( callDataIds ) )
        
    
    def getConnectedCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getConnectedCalls")
        return self.getConnectedCallsQuery(in_session).all()

    def getNumConnectedCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getNumConnectedCalls")
        return self.getConnectedCallsQuery(in_session).count()    
        
    def getConnectedInCallsQuery(self, in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getConnectedInCallsQuery")    
        return  self.getPstnInCallsFromQuery( self. \
                                              getConnectedCallsQuery(in_session))
                                              
    def getConnectedInCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getConnectedInCalls")
        return self.getConnectedInCallsQuery(in_session).all()

    def getNumConnectedInCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getNumConnectedInCalls")
        return self.getConnectedInCallsQuery(in_session).count()    

    def getConnectedOutCallsQuery(self, in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getConnectedOutCallsQuery")    
        return  self.getPstnOutCallsFromQuery( self. \
                                               getConnectedCallsQuery(in_session))

    def getConnectedOutCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getConnectedOutCalls")
        return self.getConnectedOutCallsQuery(in_session).all()

    def getNumConnectedOutCalls(self,in_session=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.getNumConnectedOutCalls")
        return self.getConnectedOutCallsQuery(in_session).count()

    def getBlockedCallTypeFromSavedQuery(self,in_session,
                                         inout_savedquery,
                                         blockedCallsQuery,
                                         in_strBlockeCallType):
         if  inout_savedquery is None:
             q1 = in_session.query(CallParameter.m_callEventId).\
                  filter(CallParameter.m_name == "reason" and
                         CallParameter.m_value.contains(in_strBlockeCallType))
             q2 = in_session.query(CallDataEvent.m_callDataId).filter(CallDataEvent.m_id.in_(q1))
             inout_savedquery = blockedCallsQuery.filter(CallData.m_id.in_(q2))
         else:
             inout_savedquery.populate_existing()
         return  inout_savedquery
             
        
    
    def getBlockedCallTypeFromQuery(self,in_session,blockedCallsQuery,in_strBlockeCallType):

        if in_strBlockeCallType == "BUSY":
            return self.getBlockedCallTypeFromSavedQuery(in_session,
                                                         self.m_busyBlockedCallsQuery,
                                                         blockedCallsQuery,
                                                         in_strBlockeCallType)
        elif in_strBlockeCallType == "NO_ANSWER":
            return self.getBlockedCallTypeFromSavedQuery(in_session,
                                                         self.m_noAnswerBlockedCallsQuery,
                                                         blockedCallsQuery,
                                                         in_strBlockeCallType)
        elif in_strBlockeCallType == "INVALID_NUMBER":
            return self.getBlockedCallTypeFromSavedQuery(in_session,
                                                         self.m_notFoundBlockedCallsQuery,
                                                         blockedCallsQuery,
                                                         in_strBlockeCallType)
    def getDisconnectedEventsInIntervalCallDataIdQuery(self,
                                                       intervalStart,
                                                       in_session):
        return in_session.query(CallDataEvent.m_callDataId).\
               filter(CallDataEvent.m_newState == "DISCONNECTED").\
               filter(CallDataEvent.m_timestamp >= intervalStart )

    def getConnectedEventsInIntervalCallDataIdQuery(self,in_session):
        now = datetime.datetime.now()           
        return in_session.query(CallDataEvent.m_callDataId).\
               filter(CallDataEvent.m_newState == "CONNECTED").\
               filter( CallDataEvent.m_timestamp <= now)


    def remoteInactiveCallDataOlderThan(self, in_date=None):
        __pychecker__ = 'unusednames=t'
        t = TraceScope(fileScopeLogger, "GWCallDataRegistry.remoteInactiveCallDataOlderThan")
        try:
            # default delete inactive call that ended before now
            if in_date is None:
                in_date = datetime.datime.now
            
            log_debug( fileScopeLogger,
                       "removing inactive calls older than %s",
                       in_date )
            session = self.getNewDBSession()
            allCallsQuery = self.getInactiveCallsQuery(session)
            callsInTimeRangeQuery = self. \
                                    getCallsInTimeRangeFromQuery( callsWithDirectionQuery,
                                                                  None,
                                                                  options.endTime)
            calls = None
            if options.maxNumCalls is None:
                calls = callsInTimeRangeQuery.all()
            else:
                options.maxNumCalls = -1 * options.maxNumCalls
                calls = callsInTimeRangeQuery. \
                        order_by(CallData.m_startTimestamp)[options.maxNumCalls:]                
            for call in calls:
                session.delete(call)
            session.commit()
            log_debug( fileScopeLogger, "removed inactive calls" )
            
        except dbexcep.DBAPIError, e:
            log_error(fileScopeLogger, "failed to remove inactive calls\n" )
            session.rollback()

        return True;
    
        
        
#------------------------- INBOUND CALLS STATISTICS ----------------------------
    def getInboundCallsIntervalStats(self,
                                     intervalStart,
                                     in_session,
                                     callDataInitiatedInInterval,
                                     callDataEventsInInterval,
                                     callDataTerminatedInInterval,
                                     activeCallsInInterval,
                                     out_dict):
        
        inboundCallDataInitiatedInInterval = \
                                           self.getSameDirectionDataFromQuery(callDataInitiatedInInterval,
                                                                              "PSTN IN->")
        inboundCallDataTerminatedInInterval = \
                                            self.getSameDirectionDataFromQuery(callDataTerminatedInInterval,
                                                                              "PSTN IN->")

        gwNumInboundCalls = inboundCallDataInitiatedInInterval.count()
        out_dict["gwNumInboundCalls"] = gwNumInboundCalls

        #all currently active calls
        gwNumConnectedInboundCalls = activeCallsInInterval.\
                                     filter(CallData.m_direction.contains("PSTN IN->")).\
                                     filter(CallData.m_state == "CONNECTED").count()

        #plus all calls that are not currently active and that once got active in this interval.
        query_1 = self.getDisconnectedEventsInIntervalCallDataIdQuery(intervalStart,
                                                                      in_session)
        query_2 = self.getConnectedEventsInIntervalCallDataIdQuery(in_session)
        gwNumConnectedInboundCalls = gwNumConnectedInboundCalls + in_session.query(CallData).\
                                     filter(CallData.m_direction.contains("PSTN IN->")).\
                                     filter(and_(CallData.m_id.in_(query_1),CallData.m_id.in_(query_2))).count()
        
        out_dict["gwNumConnectedInboundCalls"] = gwNumConnectedInboundCalls


        blockedCallsQuery = inboundCallDataTerminatedInInterval.\
                            filter(CallData.m_state == "BLOCKED")

        gwNumBlockedInboundCalls = blockedCallsQuery.count()
        out_dict["gwNumBlockedInboundCalls"] = gwNumBlockedInboundCalls

        gwNumBusyBlockedInboundCalls = 0
        gwNumNoAnswerBlockedInboundCalls = 0
        gwNumNotFoundBlockedInboundCalls = 0
        gwNumOtherFailureBlockedInboundCalls = 0
        
        gwNumBusyBlockedInboundCalls = self.getBlockedCallTypeFromQuery(in_session,
                                                                        blockedCallsQuery,
                                                                        "BUSY").count()
        gwNumNoAnswerBlockedInboundCalls = self.getBlockedCallTypeFromQuery(in_session,
                                                                            blockedCallsQuery,
                                                                            "NO_ANSWER").count()
        gwNumNotFoundBlockedInboundCalls = self.getBlockedCallTypeFromQuery(in_session,
                                                                            blockedCallsQuery,
                                                                            "INVALID_NUMBER").count()
        gwNumOtherFailureBlockedInboundCalls =  gwNumBlockedInboundCalls - \
                                               (gwNumBusyBlockedInboundCalls + \
                                                gwNumNoAnswerBlockedInboundCalls + \
                                                gwNumNotFoundBlockedInboundCalls)
        
        out_dict["gwNumBusyBlockedInboundCalls"] = gwNumBusyBlockedInboundCalls
        out_dict["gwNumNoAnswerBlockedInboundCalls"] = gwNumNoAnswerBlockedInboundCalls
        out_dict["gwNumNotFoundBlockedInboundCalls"] = gwNumNotFoundBlockedInboundCalls
        out_dict["gwNumOtherFailureBlockedInboundCalls"] = gwNumOtherFailureBlockedInboundCalls


        #statistics initial values
        gwMaxConcurrentInboundCalls = activeCallsInInterval.filter(CallData.m_direction.contains("PSTN IN->")).count()
        gwMinConcurrentInboundCalls = activeCallsInInterval.filter(CallData.m_direction.contains("PSTN IN->")).count()

        inboundCallDataEventsInInterval = \
                                        callDataEventsInInterval.\
                                        filter(CallDataEvent.m_direction.contains("PSTN IN->"))
        inboundCallDataEventsInIntervalList = inboundCallDataEventsInInterval.all()

        for event in inboundCallDataEventsInIntervalList:
          #get all active calls at the time this event happens...
          activeCalls = in_session.query(CallData).\
                        filter(CallData.m_direction.contains("PSTN IN->")).\
                        filter(and_(or_(CallData.m_endTimestamp == None, CallData.m_endTimestamp >= event.m_timestamp),
                                    CallData.m_startTimestamp <= event.m_timestamp)).count()
          log_debug(fileScopeLogger, "Number of inbound active calls for event with time stamp %s is %s ",
                    event.m_timestamp,activeCalls)
          #update the statistics...
          if activeCalls > gwMaxConcurrentInboundCalls:
             gwMaxConcurrentInboundCalls = activeCalls
          elif activeCalls < gwMinConcurrentInboundCalls:
            gwMinConcurrentInboundCalls = activeCalls

        out_dict["gwMaxConcurrentInboundCalls"] = gwMaxConcurrentInboundCalls
        out_dict["gwMinConcurrentInboundCalls"] = gwMinConcurrentInboundCalls


        inboundCallsInitiatedAndTerminatedInInterval = \
                                                     inboundCallDataInitiatedInInterval.\
                                                     filter(CallData.m_active == False).all()
        gwNumLessThan6sInboundCalls = 0
        gwNumLessThan15sInboundCalls = 0
        gwNumLessThan30sInboundCalls = 0
        gwNumLessThan60sInboundCalls = 0
        for callData in inboundCallsInitiatedAndTerminatedInInterval:

          if  callData.m_endTimestamp - callData.m_startTimestamp <= \
                 datetime.timedelta(seconds=6):
            gwNumLessThan6sInboundCalls = gwNumLessThan6sInboundCalls + 1
            gwNumLessThan15sInboundCalls = gwNumLessThan15sInboundCalls + 1
            gwNumLessThan30sInboundCalls = gwNumLessThan30sInboundCalls + 1
            gwNumLessThan60sInboundCalls = gwNumLessThan60sInboundCalls + 1

          elif  callData.m_endTimestamp - callData.m_startTimestamp <= \
                 datetime.timedelta(seconds=15):
            gwNumLessThan15sInboundCalls = gwNumLessThan15sInboundCalls + 1
            gwNumLessThan30sInboundCalls = gwNumLessThan30sInboundCalls + 1
            gwNumLessThan60sInboundCalls = gwNumLessThan60sInboundCalls + 1

          elif  callData.m_endTimestamp - callData.m_startTimestamp <= \
                 datetime.timedelta(seconds=30):
            gwNumLessThan30sInboundCalls = gwNumLessThan30sInboundCalls + 1
            gwNumLessThan60sInboundCalls = gwNumLessThan60sInboundCalls + 1

          elif  callData.m_endTimestamp - callData.m_startTimestamp <= \
                 datetime.timedelta(seconds=60):
            gwNumLessThan60sInboundCalls = gwNumLessThan60sInboundCalls + 1

        out_dict["gwNumLessThan6sInboundCalls"] = gwNumLessThan6sInboundCalls
        out_dict["gwNumLessThan15sInboundCalls"] = gwNumLessThan15sInboundCalls
        out_dict["gwNumLessThan30sInboundCalls"] = gwNumLessThan30sInboundCalls
        out_dict["gwNumLessThan60sInboundCalls"] = gwNumLessThan60sInboundCalls

#----------------------------- OUTBOUND CALLS STATISTICS -----------------------
    def getOutboundCallsIntervalStats(self,
                                      intervalStart,
                                      in_session,
                                      callDataInitiatedInInterval,
                                      callDataEventsInInterval,
                                      callDataTerminatedInInterval,
                                      activeCallsInInterval,
                                      out_dict):

        outboundCallDataInitiatedInInterval = \
                                           self.getSameDirectionDataFromQuery(callDataInitiatedInInterval,
                                                                              "->PSTN OUT")
        outboundCallDataTerminatedInInterval = \
                                            self.getSameDirectionDataFromQuery(callDataTerminatedInInterval,
                                                                              "->PSTN OUT")

        gwNumOutboundCalls = outboundCallDataInitiatedInInterval.count()
        out_dict["gwNumOutboundCalls"] = gwNumOutboundCalls


        #all currently active calls
        gwNumConnectedOutboundCalls = activeCallsInInterval.\
                                     filter(CallData.m_direction.contains("->PSTN OUT")).\
                                     filter(CallData.m_state == "CONNECTED").count()

        #plus all calls that are not currently active and that once got active in this interval.
        query_1 = self.getDisconnectedEventsInIntervalCallDataIdQuery(intervalStart,
                                                                      in_session)
        query_2 = self.getConnectedEventsInIntervalCallDataIdQuery(in_session)
        gwNumConnectedOutboundCalls = gwNumConnectedOutboundCalls + in_session.query(CallData).\
                                      filter(CallData.m_direction.contains("->PSTN OUT")).\
                                      filter(and_(CallData.m_id.in_(query_1),CallData.m_id.in_(query_2))).count()
        
        out_dict["gwNumConnectedOutboundCalls"] = gwNumConnectedOutboundCalls


        blockedCallsQuery = outboundCallDataTerminatedInInterval.\
                            filter(CallData.m_state == "BLOCKED")

        gwNumBlockedOutboundCalls = blockedCallsQuery.count()
        out_dict["gwNumBlockedOutboundCalls"] = gwNumBlockedOutboundCalls

        gwNumBusyBlockedOutboundCalls = 0
        gwNumNoAnswerBlockedOutboundCalls = 0
        gwNumNotFoundBlockedOutboundCalls = 0
        gwNumOtherFailureBlockedOutboundCalls = 0

        gwNumBusyBlockedOutboundCalls = self.getBlockedCallTypeFromQuery(in_session,
                                                                         blockedCallsQuery,
                                                                         "BUSY").count()
        gwNumNoAnswerBlockedOutboundCalls = self.getBlockedCallTypeFromQuery(in_session,
                                                                             blockedCallsQuery,
                                                                             "NO_ANSWER").count()
        gwNumNotFoundBlockedOutboundCalls = self.getBlockedCallTypeFromQuery(in_session,
                                                                             blockedCallsQuery,
                                                                             "INVALID_NUMBER").count()
        gwNumOtherFailureBlockedOutboundCalls =  gwNumBlockedOutboundCalls - \
                                               (gwNumBusyBlockedOutboundCalls + \
                                                gwNumNoAnswerBlockedOutboundCalls + \
                                                gwNumNotFoundBlockedOutboundCalls)
        
        out_dict["gwNumBusyBlockedOutboundCalls"] = gwNumBusyBlockedOutboundCalls
        out_dict["gwNumNoAnswerBlockedOutboundCalls"] = gwNumNoAnswerBlockedOutboundCalls
        out_dict["gwNumNotFoundBlockedOutboundCalls"] = gwNumNotFoundBlockedOutboundCalls
        out_dict["gwNumOtherFailureBlockedOutboundCalls"] = gwNumOtherFailureBlockedOutboundCalls


        #statistics initial values
        gwMaxConcurrentOutboundCalls = activeCallsInInterval.filter(CallData.m_direction.contains("->PSTN OUT")).count()
        gwMinConcurrentOutboundCalls = activeCallsInInterval.filter(CallData.m_direction.contains("->PSTN OUT")).count()

        outboundCallDataEventsInInterval = \
                                        callDataEventsInInterval.\
                                        filter(CallDataEvent.m_direction.contains("->PSTN OUT"))
        outboundCallDataEventsInIntervalList = outboundCallDataEventsInInterval.all()

        for event in outboundCallDataEventsInIntervalList:
          #get all active calls at the time this event happens...
          activeCalls = in_session.query(CallData).\
                        filter(CallData.m_direction.contains("->PSTN OUT")).\
                        filter(and_(or_(CallData.m_endTimestamp == None, CallData.m_endTimestamp >= event.m_timestamp),
                                    CallData.m_startTimestamp <= event.m_timestamp)).count()
          log_debug(fileScopeLogger, "Number of outbound active calls for event with time stamp %s is %s ",
                    event.m_timestamp,activeCalls)
          #update the statistics...
          if activeCalls > gwMaxConcurrentOutboundCalls:
             gwMaxConcurrentOutboundCalls = activeCalls
          elif activeCalls < gwMinConcurrentOutboundCalls:
            gwMinConcurrentOutboundCalls = activeCalls

        out_dict["gwMaxConcurrentOutboundCalls"] = gwMaxConcurrentOutboundCalls
        out_dict["gwMinConcurrentOutboundCalls"] = gwMinConcurrentOutboundCalls


        outboundCallsInitiatedAndTerminatedInInterval = \
                                                     outboundCallDataInitiatedInInterval.\
                                                     filter(CallData.m_active == False).all()
        gwNumLessThan6sOutboundCalls = 0
        gwNumLessThan15sOutboundCalls = 0
        gwNumLessThan30sOutboundCalls = 0
        gwNumLessThan60sOutboundCalls = 0
        for callData in outboundCallsInitiatedAndTerminatedInInterval:

          if  callData.m_endTimestamp - callData.m_startTimestamp <= \
                 datetime.timedelta(seconds=6):
            gwNumLessThan6sOutboundCalls = gwNumLessThan6sOutboundCalls + 1
            gwNumLessThan15sOutboundCalls = gwNumLessThan15sOutboundCalls + 1
            gwNumLessThan30sOutboundCalls = gwNumLessThan30sOutboundCalls + 1
            gwNumLessThan60sOutboundCalls = gwNumLessThan60sOutboundCalls + 1

          elif  callData.m_endTimestamp - callData.m_startTimestamp <= \
                 datetime.timedelta(seconds=15):
            gwNumLessThan15sOutboundCalls = gwNumLessThan15sOutboundCalls + 1
            gwNumLessThan30sOutboundCalls = gwNumLessThan30sOutboundCalls + 1
            gwNumLessThan60sOutboundCalls = gwNumLessThan60sOutboundCalls + 1

          elif  callData.m_endTimestamp - callData.m_startTimestamp <= \
                 datetime.timedelta(seconds=30):
            gwNumLessThan30sOutboundCalls = gwNumLessThan30sOutboundCalls + 1
            gwNumLessThan60sOutboundCalls = gwNumLessThan60sOutboundCalls + 1

          elif  callData.m_endTimestamp - callData.m_startTimestamp <= \
                 datetime.timedelta(seconds=60):
            gwNumLessThan60sOutboundCalls = gwNumLessThan60sOutboundCalls + 1

        out_dict["gwNumLessThan6sOutboundCalls"] = gwNumLessThan6sOutboundCalls
        out_dict["gwNumLessThan15sOutboundCalls"] = gwNumLessThan15sOutboundCalls
        out_dict["gwNumLessThan30sOutboundCalls"] = gwNumLessThan30sOutboundCalls
        out_dict["gwNumLessThan60sOutboundCalls"] = gwNumLessThan60sOutboundCalls



#----------------------------- CALLS STATISTICS --------------------------------


#------------------------- ACTIVE CALLS STATISTICS ----------------------------
    def getActiveCallsStatistics(self,in_session=None):
        if in_session is None:
            session = self.getNewDBSession()
        else:
            session = in_session
            
        out_dict = {}
        out_dict["gwNumActivePstnInCalls"] = self.getNumActivePstnInCalls(session)
        out_dict["gwNumActivePstnOutCalls"] =  self.getNumActivePstnOutCalls(session)
        out_dict["gwNumActiveLongCalls"] =  self.getNumActiveLongCalls(session)
        
        log_debug(fileScopeLogger, "ACTIVE STATS: %s ",out_dict)
        return out_dict

#----------------------------- INTERVAL CALLS STATISTICS --------------------------------
    def getCurrentIntervalStats(self,
                                in_strIntervalStartTime,
                                in_session):
        return self.getIntervalStats(in_strIntervalStartTime,None,in_session)
    
    def getIntervalStats(self,
                         in_strIntervalStartTime,
                         in_strIntervalLenghtSeconds = None,
                         in_session=None):

        intervalStart = datetime.datetime.\
                        strptime(in_strIntervalStartTime[:19],"%Y-%m-%d %H:%M:%S")

        if in_strIntervalLenghtSeconds is None:
            td = datetime.datetime.now() -  intervalStart
            strIntervalLenghtSeconds = str(td.seconds)
        else:
            strIntervalLenghtSeconds = in_strIntervalLenghtSeconds

        if in_session is None:
            session = self.getNewDBSession()
        else:
            session = in_session
            
        out_dict = {}

        log_debug(fileScopeLogger, "Retrieving stats for interval started %s with lenght %s seconds",
                  in_strIntervalStartTime,
                  strIntervalLenghtSeconds)

        callDataInitiatedInInterval = \
                                    self.getDataInitiatedInInterval(in_strIntervalStartTime,
                                                                    strIntervalLenghtSeconds,
                                                                    "CallData",
                                                                     session)
        log_debug(fileScopeLogger, "callDataInitiatedInInterval: %s ", callDataInitiatedInInterval.count())
        callDataEventsInInterval = \
                                 self.getDataInitiatedInInterval(in_strIntervalStartTime,
                                                                 strIntervalLenghtSeconds,
                                                                 "CallDataEvents",
                                                                  session)
        log_debug(fileScopeLogger, "callDataEventsInInterval: %s ",  callDataEventsInInterval.count())
        callDataTerminatedInInterval = \
                                      self.getCallDataTerminatedInInterval(in_strIntervalStartTime,
                                                                           strIntervalLenghtSeconds,
                                                                           session)
        log_debug(fileScopeLogger, "callDataTerminatedInInterval: %s ", callDataTerminatedInInterval.count())

        #all currently active calls are considered as active calls in this interval...
        activeCallsInInterval =  session.query(CallData).filter(CallData.m_active == True)
        log_debug(fileScopeLogger, "activeCallsInInterval: %s ", activeCallsInInterval.count())

        #INBOUND CALLS STATS
        self.getInboundCallsIntervalStats(intervalStart,
                                          session,
                                          callDataInitiatedInInterval,
                                          callDataEventsInInterval,
                                          callDataTerminatedInInterval,
                                          activeCallsInInterval,
                                          out_dict)
       


        #OUTBOUND CALLS STATS
        self.getOutboundCallsIntervalStats(intervalStart,
                                           session,
                                           callDataInitiatedInInterval,
                                           callDataEventsInInterval,
                                           callDataTerminatedInInterval,
                                           activeCallsInInterval,
                                           out_dict)
        
        callDataEventsInIntervalList = callDataEventsInInterval.all()
        gwMaxConcurrentCalls = 0 #activeCallsInInterval.count()

        for event in callDataEventsInIntervalList:
          #get all active calls at the time this event happens...
          activeCalls = session.query(CallData).\
                        filter(and_(or_(CallData.m_endTimestamp == None, CallData.m_endTimestamp >= event.m_timestamp),
                                    CallData.m_startTimestamp <= event.m_timestamp)).count()
          log_debug(fileScopeLogger, "Number of active calls for event with time stamp %s is %s ",
                    event.m_timestamp,activeCalls)
          #update the statistics...
          if activeCalls > gwMaxConcurrentCalls:
              gwMaxConcurrentCalls = activeCalls
        
        out_dict["gwMaxConcurrentCalls"] = gwMaxConcurrentCalls

        log_debug(fileScopeLogger, "STATS: %s ",out_dict)

        return out_dict


if __name__ == '__main__':

    # Main
    def main(argv=None):

        if argv is None:    
            argv = sys.argv
            
        fileScopeLogger.setLogLevel(LogLevel.debug)

        dStart = datetime.datetime.now()
            
        callDataRegistry = GWCallDataRegistry()
        if callDataRegistry.configure('sqlite:///./callData.db', False ) == False:
            return 1


        # Add call data
        log_info( fileScopeLogger, "creating call1..." )
        call1 = CallData()
        call1.m_callId = '12345'
        call1.m_active = True
        if callDataRegistry.addCallData(call1) == False:
          return 2

        log_info( fileScopeLogger, "setting event1 to call1..." )
        event1 = CallDataEvent()
        event1.m_type = 'setup'
        event1.m_direction = 'PSTN IN->SIP OUT'
        event1.m_newState = 'SETUP'
        event1.setCallParameter( "pstn.in.ani", '4504417377' )
        event1.setCallParameter( "pstn.in.ani", '4504417378' )
        event1.setCallParameter( "pstn.in.dnis", '4504411234' )
        call1.pushEvent( event1 )

        event2 = CallDataEvent()
        event2.m_type = 'connect'
        event2.m_direction = 'PSTN IN->SIP OUT'
        event2.m_newState = 'CONNECTED'
        call1.pushEvent( event2 )


        # sleep a little
        sleepDuration = 5
        log_info(fileScopeLogger, "sleeping %d seconds...", sleepDuration )
        time.sleep( sleepDuration )

        # New call
        log_info( fileScopeLogger, "creating call2..." )
        call2 = CallData()
        call2.m_callId = '12346'
        call2.m_active = True
        if callDataRegistry.addCallData(call2) == False:
          return 3

        log_info( fileScopeLogger, "setting event2 to call2..." )
        event2 = CallDataEvent()
        event2.m_type = 'setup'
        event2.m_direction = 'SIP IN->PSTN OUT'
        event2.m_newState = 'SETUP'
        event2.setCallParameter( "sip.in.fromUri", 'sip:jean@paraxip.com' )
        event2.setCallParameter( "sip.in.toUri", 'sip:roger@paraxip.com' )
        call2.pushEvent( event2 )
        callDataRegistry.commit()
        
        log_info( fileScopeLogger,
                  "**** Active Pstn In calls '%s'" % callDataRegistry.getNumActivePstnInCalls() )

        log_info( fileScopeLogger,
                  "**** Active Pstn Out calls '%s'" % callDataRegistry.getNumActivePstnOutCalls() )

        log_info( fileScopeLogger,
                  "**** Active Long calls '%s'" % callDataRegistry.getNumActiveLongCalls() )


        # stop the first call
        log_info( fileScopeLogger, "*** endding call1..." )
        call1.callEnd()
        call1 = None
        
        log_info( fileScopeLogger,
                  "**** Active Pstn In calls after endding call1 '%s'" % callDataRegistry.getNumActivePstnInCalls() )

        log_info( fileScopeLogger,
                  "**** Active Pstn Out calls after endding call1 '%s'" % callDataRegistry.getNumActivePstnOutCalls() )

        log_info( fileScopeLogger,
                  "**** Active Long calls  after endding call1 '%s'" % callDataRegistry.getNumActiveLongCalls() )


        # stop the 2nd call
        log_info( fileScopeLogger, "*** endding call2..." )              
        call2.callEnd()
        call2 = None

        log_info( fileScopeLogger,
                  "**** Active Pstn In calls after endding call2 '%s'" % callDataRegistry.getNumActivePstnInCalls() )

        log_info( fileScopeLogger,
                  "**** Active Pstn Out calls  after endding call2 '%s'" % callDataRegistry.getNumActivePstnOutCalls() )

        log_info( fileScopeLogger,
                  "**** Active Long calls after endding call2 '%s'" % callDataRegistry.getNumActiveLongCalls() )


        # Cleanup load entries
        log_info( fileScopeLogger,
                  "**** Active Pstn In calls after cleanup '%s'" % callDataRegistry.getNumActivePstnInCalls() )

        log_info( fileScopeLogger,
                  "**** Active Pstn Out calls after cleanup '%s'" % callDataRegistry.getNumActivePstnOutCalls() )

        log_info( fileScopeLogger,
                  "**** Active Long calls after after cleanup '%s'" % callDataRegistry.getNumActiveLongCalls() )

        stats = callDataRegistry.getIntervalStats(str(dStart),"30")

        log_info( fileScopeLogger,
              "**** Call stats '%s'" % stats )
                            
        # Success
        return 0

    # Execute main()
    sys.exit( main() )
## !!MIRROR FILE, DO NOT MODIFY!!
