import turbogears
import simplejson
import os
import socket
import logging
import subprocess

from odict import odict
from turbogears import controllers, expose
from turbogears import flash, redirect, url
from paraxip.gateway_gui import extjs
from paraxip.gateway_gui_widgets.tabpstn  import TabPSTN
from paraxip.gateway_gui_widgets.treeviewpstn  import TreeViewPSTN
from netborderPyInfra import PyGlobalConfig as globalConfig
from netborderPyInfra import ParameterInfoDB, ParameterTrie, StringVector 
from netborderPyInfra import ParameterInfoWithName
from netborderPyInfra import *
from pstnconfig import model
from operator import itemgetter
from cgi import FieldStorage
from paraxip.smart_update.ImportCfg import importCfg
from paraxip.smart_update.ExportCfg import exportCfg
from cherrypy.lib.cptools import serve_file
from datetime import datetime

log = logging.getLogger("pstnconfig.controllers")

class ConfigController(controllers.Controller):

  def __init__ (self, in_strProjectPath, in_webServiceObj):
    log.info("Initializing config controller")
    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/"
                                         "javascript/widgets_src/editor.js")
    self.isConfigOutOfSynch = False
    self.instDir=globalConfig.getParameter("netborder.Installation.Directory")
    self.extTree = TreeViewPSTN('pstnTree', renderTo='tree', 
                                fetch='config/fetchTree', 
                                src_file=strSrcFile)
    
    #Elements that will build our tabber
    tabs = odict()
    #Each elements of the tabs dictionnary has the format:
    # tabs['Tab Text'] = ['/autoloadUrl','tabId']        
    tabs['Routing Rules'] = ['','routingRulesTabId',"Edit the routing rules"]
    tabs['PSTN Config'] = ['','pstnEditorTabId',"Edit pstn configuration"]
    tabs['Advanced'] = ['','advancedTabId',"Edit advanced configuration"]
    tabs['FXO Calibration'] = ['','fxoTuneTabId',"Calibrate FXO interface"]
    tabs['Backup/Restore'] = \
      ['','smartUpdateTabId',"Backup or Restore configuration for the gateway"]
    tabs['Provisioning'] = \
      ['','provisioningTabId',"Provision xml definition file to generate configuration for the gateway"]
    
    items = [{
      'title': k, 'autoLoad': v[0], 'id': v[1], 'tabTip': v[2]
    } for k, v in tabs.items()]

    tabList = [
      (k.replace(" ","").lower(),v[0],v[1],v[2]) for k, v in tabs.items()
    ]
    tabs = dict(tabs)
    
    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/"
                              "javascript/widgets_src/pstn_config_tabs.js")
    
    self.extSubTabs = TabPSTN('configSubTabs',tabs=tabs, items=items, 
                              tabList=tabList,
                              src_file=strSrcFile,children=[self.extTree])

    self.routingRulesPath = os.path.join(globalConfig.getParameter(
        "netborder.Installation.Directory"), "config/routing-rules.xml")
    log.info("Routing rules path: %s" % self.routingRulesPath)
    self.webService = in_webServiceObj

    self.loadProperties()


  def getSubTabs (self):
    return self.extSubTabs

  @expose()
  def index (self):
    """Displays the configuration options to the user"""

    return dict(extSubTabs=extSubTabs)

  @expose(template='pstnconfig.templates.config.routing')
  def lightbox_routing_rules (self):
    """Displays the routing rules"""
    if globalConfig.getParameter("netborder.oamWebServer.offerVirtualBoards"):
      offerVirtualBoards = "true"
    else:
      offerVirtualBoards = "false"
    if globalConfig.getParameter("netborder.oamWebServer.isGatewayInCLI"):
      isGatewayInCLI = "true"
    else:
      isGatewayInCLI = "false"

    return {
      "footerText":"",
      "offerVirtualBoards":offerVirtualBoards,
      "isGatewayInCLI":isGatewayInCLI
    }

  @expose(template='pstnconfig.templates.config.editor')
  def lightbox_pstn_config (self):
    """Displays the PSTN config tab"""
    if globalConfig.getParameter("netborder.oamWebServer.offerVirtualBoards"):
      offerVirtualBoards = "true"
    else:
      offerVirtualBoards = "false"

    if globalConfig.getParameter("netborder.oamWebServer.isGatewayInCLI"):
      isGatewayInCLI = "true"
    else:
      isGatewayInCLI = "false"

    if globalConfig.getParameter(
        "netborder.oamWebServer.enableHardwareDetection"):
      autoDetectHardware = "true"
    else:
      autoDetectHardware = "false"

    returnDict = dict(
      footerText="",
      offerVirtualBoards=offerVirtualBoards,
      isGatewayInCLI=isGatewayInCLI,
      autoDetectHardware=autoDetectHardware,
      extTree=self.extTree
    )
    return returnDict

  @expose(template='pstnconfig.templates.config.smartupdate')
  def lightbox_backup(self):

    return {"footerText":""}

  @expose(template='pstnconfig.templates.config.provisioning')
  def lightbox_provisioning(self):

    return {"footerText":""}

  @expose(template='pstnconfig.templates.config.advanced')
  def lightbox_advanced(self):

    return {"footerText":""}
	
  @expose()
  def changeConfigOutOfSynch(self, newValue):
    log.info("Changing isConfigOutOfSynch to: %s" % newValue)
    if newValue == "true":
      self.isConfigOutOfSynch = True
    else: 
      self.isConfigOutOfSynch = False
    return {}

  @expose()
  def getIsConfigOutOfSynch(self, _dc=''):
    return {'isConfigOutOfSynch':self.isConfigOutOfSynch}

  @expose()
  def getConfigStatus(self, _dc=''):
    log.info("Is Config Valid: %s" % model.isConfigValid() )
    if model.isConfigValid() : 
      if self.isConfigOutOfSynch :
        configStatus = "outOfSync"
      else:
        configStatus = "valid"      
    else:
      configStatus = "invalid"      
    return {'configStatus':configStatus}

  @expose()
  def getRoutingRules(self):
    log.info("Getting routing rules")
    try:
      f = open(self.routingRulesPath,'r')
      # Read all content
      strFileContent = f.read()
      f.close()
    except IOError:
      log.error("Cannot open file ", self.routingRulesPath)               
      return "" 
    return {'routingRulesText':strFileContent}

  @expose()
  def saveRoutingRules(self, **data):
    log.info("Saving routing rules: %s" % data)
    routingText = simplejson.loads(data['xml'])['routingRulesEditField']

    result = self.validateRoutingRules(**data)
    result = simplejson.loads(result)

    if result['isValid'] == 'true':
      try:
         f = open(self.routingRulesPath,'w')
         f.write(routingText)
         f.close()
      except IOError:
        log.error("Cannot open file ", self.routingRulesPath)               
        return {
          'isValid' : result['isValid'],
          'validationMsg' : result['errorMsg'],
          'savingMsg' : " Cannot open file %s" % self.routingRulesPath
        }
      return {
        'isValid' : result['isValid'],
        'validationMsg' : result['errorMsg'],
        'savingMsg' : ' File saved!'
      }
    else:
      return {
        'isValid' : result['isValid'],
        'validationMsg' : result['errorMsg'],
        'savingMsg' : ' File not saved!'
      }

  ##########################################################################
  # Validate the Routing rules
  # @param xml: the xml string representing the routing rules
  @expose()
  def validateRoutingRules(self, **xml):
    log.info("Routing rules string to validate: " % xml)
    rrXmlStr = simplejson.loads(xml['xml'])['routingRulesEditField']
    result = {}
    try:
      wsResult = self.webService.validateRoutingRules(rrXmlStr)
      result['m-bExecSuccess'] = wsResult['m-bExecSuccess']
      result['m-strStatusText'] = wsResult['m-strStatusText']

      if result['m-bExecSuccess'] == 'true':
        result['m-strStatusText'] = "XML file is valid."
      else:
        result['m-strStatusText'] = \
          "XML file is not valid: " + result['m-strStatusText']
    except socket.error:
      result['m-bExecSuccess'] = 'true'
      result['m-strStatusText'] = \
        "Unable to validate file. Gateway must be started in order to " + \
        "proceed. The file was saved but may be invalid."
    return {
      'isValid' : result['m-bExecSuccess'],
      'errorMsg' : result['m-strStatusText']
    }
 
  @expose()
  def editor(self):
    return dict(extTree = self.extTree)

  ##########################################################################
  # Fetch Data provider for the root Ext.tree.AsyncTreeNode in the TreeView
  # Returns a json-encoded string a tree representation of the configuration
  @expose()
  def fetchTree(self, node):
    # Load current pstnconfig model if not done already
    # TODO check for errors
    if model.currentConfig is None and model.PSTNConfigFileExists():
      model.currentConfig = model.parse(model.getPSTNConfigFileName())
      
    return simplejson.dumps(model.getJSONForEditor(model.currentConfig))

  @expose()
  def getFxoInterfaces(self):
    # Load current pstnconfig model if not done already
    # TODO check for errors
    if model.currentConfig is None and model.PSTNConfigFileExists():
      model.currentConfig = model.parse(model.getPSTNConfigFileName())
    interfaceList = []
    
    if(model.currentConfig is not None):
      boards = model.currentConfig.get_boards()
      
      interfaceList = []
      for board in boards.get_sangomaAnalogBoard():
        for interface in board.get_interface():
          if interface.get_type() == "FXO" :
            interfaceList.append({
              'boardName':board.get_displayName(),
              'wanpipe':board.get_wanpipe(),
              'wanpipeInterface':interface.get_wanpipeInterface()
            })
  
    return(str(extjs.FormActionLoadResponse(success=True, data=interfaceList)))

  def loadProperties(self):
    # Load parameter DB
    self.strParamDBFile = os.path.join(self.instDir,
        "config/netborder-gateway-paramDefs.xml")
    log.debug( "loading parameter DB from file: " +
               self.strParamDBFile);
    self.paramDB = ParameterInfoDB.loadFromXmlFile(self.strParamDBFile);
    if self.paramDB is None :
      log.error("failed to load parameter DB from file: " +
                self.strParamDBFile )
    # Load input file
    self.inputFilePropertiesFiles = os.path.join(self.instDir,
        "config/gw.properties")
    log.debug( "reading input properties in file(s): %s" % \
               self.inputFilePropertiesFiles )
    vInputFilePropertiesFiles = StringVector()
    vInputFilePropertiesFiles.append( self.inputFilePropertiesFiles )    

    self.paramTrie = ParameterTrie()
    if self.paramTrie.load(vInputFilePropertiesFiles, self.paramDB) != True :
        log.error("failed to load properties file(s): " +
                  self.inputFilePropertiesFiles )

    self.origParamTrie = ParameterTrie()
    if self.origParamTrie.load(vInputFilePropertiesFiles, self.paramDB)!=True :
        log.error("failed to load properties file(s): " +
                  self.inputFilePropertiesFiles )
     
    # Applying default values
    log.debug( "adding properties with default values" )
    self.paramDB.addDefaultsToConfig(self.paramTrie, False )

    outputFilePropertiesFiles = os.path.join(self.instDir,
        "config/gw3.properties")

    return True

  @expose()
  def getAdvancedProperties(self):
    log.debug("getAdvancedProperties")
    # Reload Properties from file
    self.loadProperties()      

    data = []
    paramInfoList = []
    # Get delta between ParamDB and Properties file
    self.paramDB.retrieve(".*", paramInfoList)

    for pi in paramInfoList :
      #print "\npi ", '\n', pi
      tmpVector = StringVector()
      isconfigured = self.origParamTrie.retrieve(pi.name,tmpVector)
      pv = self.paramTrie.getParameter(pi.name)
      #print "\nvalue: ", pv
      log.debug("adding parameter %s with value %s" % (pi.name, pv))
      data.append({'configured':isconfigured,
                   'type':str(pi.type),
                   'name':pi.name,
                   'value':pv,
                   'description':pi.description
                   })
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  @expose()
  def saveAdvancedProperties(self, properties):
    log.debug("saving properties in file:" + self.strParamDBFile)
    data = []
    param = Parameter()
    for prop in simplejson.loads(properties):
      name = str(prop[0])
      value = str(prop[1])
      result = param.initFromText( name, value, self.paramDB )
      if result != SUCCESS_INITRESULT:
        if result == PARAM_NOT_IN_DB_INITRESULT:                
          log.error("failed to set parameter "+name+" to value "+\
                      value+" because the specified parameter does "\
                      "not exist in the parameter DB");
        elif result == PARAM_WRONG_TYPE_INITRESULT :
          log.error("failed to set parameter "+name+" to value "+\
                      value+" because the specified parameter value "\
                      "type is wrong")
            #elif result == PARSE_ERROR_INITRESULT:
        else:
          log.error("failed to set parameter "+name+" to value "+\
                      value+" because the parameter value is missing")
          
          return(str(extjs.FormActionLoadResponse(success=False,data=data)))

      # Set the parameter in the trie
      if self.paramTrie.setParameter(name,value) == False:
        log.error( "failed to set parameter "+argv[0] );
        return(str(extjs.FormActionLoadResponse(success=False,data=data)))

    if self.paramTrie.saveAsPropertiesFile \
        (self.inputFilePropertiesFiles, self.paramDB, True) == False:
      log.error("failed to save output properties to file: " + 
                self.inputFilePropertiesFiles)
      return(str(extjs.FormActionLoadResponse(success=False,data=data)))
    
    return(str(extjs.FormActionLoadResponse(success=True,data=data)))

  @expose()
  def importConfiguration(self, **args):
    """Import the given configuration archive in the installation directory
    of the gateway"""
    log.info("args: %s" % args)
    archive = args['archiveFile']
    uploadDir = os.path.join(
      self.instDir,
      "config-archives"
    )

    (filename,extension) = os.path.splitext(archive.filename)
    if not extension == ".zip":
      return(str(extjs.FormActionLoadResponse(success=False,data=[{"errorMsg":"The import configuration feature works only with a zip archive of the Netborder Express configuration."}])))

    if not os.path.isdir(uploadDir) :
      os.makedirs(uploadDir)

    if not archive.file:
      log.error("archive %s has no file" % archive)
      return (str(extjs.FormActionSubmitResponse(success=False)))

    uploadedFile = os.path.join(uploadDir, archive.filename)
    fout = file(uploadedFile, 'wb')
    while 1:
        chunk = archive.file.read(100000)
        if not chunk: break
        fout.write (chunk)
    fout.close()

    configPath = os.path.join(self.instDir,"config")
    binPath = os.path.join(
      globalConfig.getParameter("netborder.Installation.Bin.Directory"),
      "netborder-smart-update"
    )

# @return 100 : standard application failure error code
# @return 101 : update succeeded with warnings
# @return 102 : invalid config file
# @return 103 : export.txt is missing from configuration archive
# @return 104 : configuration comes from a different platform (Linux or Microsoft)
# @return 105 : version of the configuration is greater then the
#               currently installed application version.
# @return 106 : currently installed version of the application is more then
#               one major version of the configuration being imported
# @return 107 : brand is not matching
# @return 108 : sub_brand is not matching
    errorMsg = [
      "Import Application failed.",
      "Update succeeded with warnings."
      "invalid config file.",
      "File export.txt is missing from configuration archive.",
      "Configuration comes from a different platform (Linux or Microsoft).",
      "Unable to get application information.",
      "Version of the configuration is greater then the currently installed application version.",
      "Currently installed version of the application is more then one major version of the configuration being imported "
      "Brand is not matching.",
      "Sub brand is not matching.",
    ]

    args = [
      binPath,
      "-i",
      "-o",
      configPath,
      "-a",
      uploadedFile,
      "--gl",
      os.path.join(configPath,"smartUpdate.properties")
    ]

    p = subprocess.Popen(
      args=args,
      stdin=subprocess.PIPE,
      stdout=subprocess.PIPE,
      stderr=subprocess.PIPE
    )

    stdout, stderr = p.communicate()

    if stderr:
      log.error("Configuration import failed: %s" % stderr)
      return(str(extjs.FormActionSubmitResponse(success=False,errors={"errorMsg":errorMsg[0]})))

    rc = 0
    rc = p.returncode
    log.debug("SmartUpdate import process return code=%s" % rc)

    if not rc == 0:
      return(str(extjs.FormActionSubmitResponse(success=False,errors={"errorMsg":errorMsg[rc-100]})))

    model.currentConfig = model.parse(model.getPSTNConfigFileName())

    return (str(extjs.FormActionSubmitResponse(success=True)))

  @expose()
  def exportConfiguration(self, js=""):
    exportPath = \
      os.path.join(
        self.instDir,
        "config"
      )

    outDir = os.path.join(self.instDir,"config-archives")
    binDir = \
      globalConfig.getParameter("netborder.Installation.Bin.Directory")
    appExe = os.path.join(binDir, "netborder-gateway")

    rc = exportCfg(exportPath, outDir, appExe, self.instDir)

    errorMsg = [
      "Export Application failure"
    ]
    succeed = False

    if not rc == 0:
      return(str(extjs.FormActionLoadResponse(success=succeed,data=[{"errorMsg":errorMsg[rc-200]}])))

    return (str(extjs.FormActionSubmitResponse(success=True)))

  @expose()
  def downloadExportedConfig(self):
    outDir = os.path.join(self.instDir,"config-archives")
    fileList = [(os.path.getmtime(os.path.join(outDir,x)),x) for x in os.listdir(outDir)]
    fileList.sort()
    path = os.path.join(outDir,fileList[-1][-1])
    return serve_file(path, "application/force-download", "attachment")


  @expose()
  def generatePbxConfig(self, **args):
    log.info("args: %s" % args)
    pbxCfgFile = args['pbxCfgFile']
    uploadDir = os.path.join(self.instDir, "tmp")

    if not os.path.isdir(uploadDir) :
      os.makedirs(uploadDir)

    if not pbxCfgFile.file:
      log.error("no files uploaded")
      return (str(extjs.FormActionSubmitResponse(success=False)))

    uploadedFile = os.path.join(uploadDir, pbxCfgFile.filename)
    fout = file(uploadedFile, 'wb')
    while 1:
        chunk = pbxCfgFile.file.read(100000)
        if not chunk: break
        fout.write (chunk)
    fout.close()

    configPath = os.path.join(self.instDir,"config")
    binPath = os.path.join(
      globalConfig.getParameter("netborder.Installation.Bin.Directory"),
      "netborder-pbx-configurator"
    )

    errorMsg = [
      "PBX Configurator process completed successfully.",
      "PBX Configurator process failed. See PbxConfigurator.log for more details.",
      "PBX Configurator input file is invalid.",
      "PBX Configurator input file version is invalid."
    ]

    args = [
      binPath,
      "-i",
      uploadedFile,
      "-o",
      configPath,
      "--gl",
      os.path.join(configPath,"pbxConfigurator.properties")
    ]

    p = subprocess.Popen(
      args=args,
      stdin=subprocess.PIPE,
      stdout=subprocess.PIPE,
      stderr=subprocess.PIPE
    )

    stdout, stderr = p.communicate()

    if stderr:
      log.error("Configuration generation failed: %s" % stderr)
      return (str(extjs.FormActionSubmitResponse(success=False)))

    rc = p.returncode
    log.info("PBX Configurator process return code=%s" % rc)

    if not rc == 0:
      return(str(extjs.FormActionSubmitResponse(success=False,errors={"errorMsg":errorMsg[rc]})))

    os.remove(uploadedFile)

    model.currentConfig = model.parse(model.getPSTNConfigFileName())
    
    return (str(extjs.FormActionSubmitResponse(success=True)))
