import logging

import turbogears
import os
import SOAPpy

log = logging.getLogger("pstnconfig.controllers")

class WebServicesController():
  def __init__(self, url, namespace):
    log.info("Initializing webservices controller")

    #Create the Proxy between Client and Server
    self.serverConnection = SOAPpy.SOAPProxy(url, namespace)

    # Enable debug logging 
    #self.serverConnection.config.debug = 1

    #Enable/Disable writting of the XML Request
    #Enable Outgoing Request
    #self.serverConnection.config.dumpSOAPOut = 1
    #Disable Outgoing Request
    self.serverConnection.config.dumpSOAPOut = 0
    #Enable Inbound Request
    #self.serverConnection.config.dumpSOAPIn = 1
    #Disable Inbound Request
    self.serverConnection.config.dumpSOAPIn = 0

##########################################################################
# executeCommand requests
##########################################################################
  ##########################################################################
  # Parameter: get-log-level logger-name 
  # Desc: Ability to ret dynamically the properties of a particular logger.
  # The log levels are: 
  # FATAL_LOG_LEVEL, 
  # ERROR_LOG_LEVEL, 
  # WARN_LOG_LEVEL, 
  # INFO_LOG_LEVEL, 
  # DEBUG_LOG_LEVEL, 
  # TRACE_LOG_LEVEL
  # Returned value: bool success/failure, string log-level
  def getLogLevel(self, loggerName):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','get-log-level')
    oamCmdStruct._addItem('m-argVector', loggerName)
    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})

  ##########################################################################
  # Parameter: set-log-level log-level logger-name 
  # Desc: Ability to modify dynamically the properties of a particular logger.
  # The log levels are: 
  # FATAL_LOG_LEVEL, 
  # ERROR_LOG_LEVEL, 
  # WARN_LOG_LEVEL, 
  # INFO_LOG_LEVEL, 
  # DEBUG_LOG_LEVEL, 
  # TRACE_LOG_LEVEL
  # Returned value: bool success/failure, string status
  def setLogLevel(self, logLevel, loggerName):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','set-log-level')
    oamCmdStruct._addItem('m-argVector',logLevel)
    oamCmdStruct._addItem('m-argVector',loggerName)
    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})


   ##########################################################################
  # Parameter: set-run-mode run-mode
  # Desc: Ability to modify dynamically the run mode.
  # The run modes are: 
  # production, 
  # development, 
  # Returned value: bool success/failure, string status
  def setRunMode(self, runMode):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','set-run-mode')
    oamCmdStruct._addItem('m-argVector',runMode)
    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})

   ##########################################################################
  # Parameter: get-run-mode
  # Desc: Ability to modify dynamically the run mode.
  # The run modes are: 
  # production, 
  # development, 
  # Returned value: bool success/failure, string status
  def getRunMode(self):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','get-run-mode')
    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})


   ##########################################################################
  # Parameter: set-call-logger-status call-logger-status
  # Desc: Ability to modify dynamically the call logger status.
  # The status can be: 
  # enabled, 
  # disabled, 
  # Returned value: bool success/failure, string status
  def setCallLoggerStatus(self, status):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','set-call-logger-status')
    oamCmdStruct._addItem('m-argVector',status)
    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})

   ##########################################################################
  # Parameter: get-call-logger-status
  # Desc: Ability to modify dynamically the call logger status.
  # The status can be: 
  # enabled, 
  # disabled, 
  # Returned value: bool success/failure, string status
  def getCallLoggerStatus(self):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','get-call-logger-status')
    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})


  

  ##########################################################################
  # Parameter: reset-alarm-state   
  # Desc: Resets the alarm status to 'Green'
  # Returned value: bool success/failure, string status
  def resetAlarmState(self):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','reset-alarm-state')

    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})

  ##########################################################################
  # Parameter: getAlarms   
  # Desc: Returns the alarms of the system that are ON
  # Returned value: List of all the alarms that are ON
  def getAlarms(self):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','get-alarms')

    oamCmdStruct = self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})
    return oamCmdStruct['m-strStatusText']

  ##########################################################################
  # Parameter: pstn-channel-op quiesce|oos|is [channel-name-regexp]
  # Desc: Set the channel state according to parameters:
  # - 'quiesce' will gently bring the selected channels out of service 
  #     after the current calls are over, 
  # - 'oos' will abruptly bring the selected channels out of service, 
  # - 'is' will bring back a channel in service, 
  # - 'channel-name-regexp' is an optional regular expression to identify 
  #   the channels on which the operation should be performed (e.g. of 
  #   the type B[1-2]T.*)
  # Returned value: bool success/failure, string status
  def setChannelState(self, state, chanRegExp=''):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','pstn-channel-op')
    oamCmdStruct._addItem('m-argVector',state)
    #TODO - Solve the array problem in SOAP when two m-argVector are passed
    #       and add the chanRegExp to the oamCmdStruct

    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})

  ##########################################################################
  # Parameter: reset-alarm-state   
  # Desc: Resets the alarm status to 'Green'
  # Returned value: bool success/failure, string status
  def validateRoutingRules(self,routingRulesXML):
    oamCmdStruct = SOAPpy.structType()
    oamCmdStruct._addItem('m-strName','validate-routingrules')
    oamCmdStruct._addItem('m-argVector',routingRulesXML)

    return self.serverConnection.executeCommand({'in-cmd':oamCmdStruct})

##########################################################################
# getOM requests
##########################################################################
  ##########################################################################
  # Parameter: netborder.om.OAMManager.hostname 
  # Desc: Get the machine hostname
  def getHostname(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.OAMManager.hostname"})

  ##########################################################################
  # Parameter: netborder.om.taskManager.state 
  # Desc: Get the gateway state 
  # Returned value: STOPPED, STARTING, RUNNING, STOPPING 
  def getGWState(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.taskManager.state"})

  ##########################################################################
  # Parameter: netborder.om.OAMManager.alarmState 
  # Desc: Get the gateway alarm state
  # Returned value: GREEN, YELLOW, ORANGE, RED
  def getGWAlarmState(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.OAMManager.alarmState"})

  ##########################################################################
  # Parameter: netborder.om.OAMManager.lastAlarm 
  # Desc: Get the last alarm
  def getLastAlarm(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.OAMManager.lastAlarm"})

  ##########################################################################
  # Parameter: netborder.om.gw.CallEngine.numStartedCall 
  # Desc: Get the number of calls received
  def getNumStartedCall(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.gw.CallEngine.numStartedCall"})

  ##########################################################################
  # Parameter: netborder.om.gw.CallEngine.numActiveCall   
  # Desc: Get the number of current calls 
  def getNumActiveCall(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.gw.CallEngine.numActiveCall"})

  ##########################################################################
  # Parameter: netborder.om.gw.CallEngine.maxNumActiveCall
  # Desc: Get the maximum number of active call since the gateway was started.
  def getMaxNumActiveCall(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.gw.CallEngine.maxNumActiveCall"})

  ##########################################################################
  # Parameter: netborder.om.gw.CallEngine.avgCallDuration
  # Desc: Get the average call duration since the gateway was started.
  def getAvgCallDuration(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.gw.CallEngine.avgCallDuration"})
  
  ##########################################################################
  # Parameter: netborder.om.gw.CallEngine.minCallDuration
  # Desc: Get the shortest call duration since the gateway was started.
  def getMinCallDuration(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.gw.CallEngine.minCallDuration"})
  
  ##########################################################################
  # Parameter: netborder.om.gw.CallEngine.maxCallDuration
  # Desc: Get the longest call duration since the gateway was started.
  def getMaxCallDuration(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.gw.CallEngine.maxCallDuration"})

  ##########################################################################
  # Parameter: netborder.om.gw.RoutingRules
  # Desc: Get the routing rules as a string.
  def getRoutingRules(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.gw.RoutingRules"})

  ##########################################################################
  #Parameter: netborder.om.pstn.gwNetIf.channelStatus
  #Desc: Get all channel state
  #Return value: a string where each line is formatted as follow:
  #  <Channel display name> SP <Channel Status> CRLF
  #SP: is a single space
  #CRLF: is a carriage-return line feed
  #Where:
  #Channel display name: the name of the channel as displayed in
  #the web ui (e.g. B1-T1),
  #
  #Channel status: any of the following:
  #INITIALIZING
  #ALARMED
  #IDLE
  #ANSWERING
  #CONNECTED
  #DIALING
  #RINGING
  #HANGING-UP
  #GOING-OUT-OF-SERVICE
  #OUT-OF-SERVICE
  #OUT-OF-SERVICE(FINAL)
  def getChannelStatus(self):
    return self.serverConnection.getOM({"in-strOmName":"netborder.om.pstn.gwNetIf.channelStatus"})
  

