Ext.namespace("Paraxip.config.fxoTune");

var validateFxoInterfacesSelected = function () {
  if(fxoInterfacesGrid.getSelectionModel().getSelections().length <= 0){
    //verify if at leat one interface is selected
    Ext.Msg.alert(
      'Start Calibration',
      'In order to execute the calibration, you must select at least one ' +
      'interface.'
    );
    return false;
  }
  return true;
}

/**
 *
 * This method determines wich form to show to the user in the wizard
 * depending on the direction and the types of boards in the grid.
 * @param direction: helps to know wich form to show; ('next' or 'back')
 *
 */
var fxoTuningNavHandler = function(direction) {
  var lay = fxoTuneWizard.getLayout();

  var active_id = lay.activeItem.id;

  var next_item;

  var show_next = true;
  var show_back = true;
  var show_runCalibration=false;
  var show_cancelCalibration=false;
  
  //Determine witch form to render_next
  if (direction == "next") { //Go forward
    switch (active_id) {
    case "Paraxip.config.fxotune.front_form":
      next_item = "Paraxip.config.fxotune.select_lines";
      break;
    case "Paraxip.config.fxotune.select_lines":
      if( validateFxoInterfacesSelected() ) {
        next_item = "Paraxip.config.fxotune_run_calibration";
        break;
      }
      next_item = "Paraxip.config.fxotune.select_lines";
      break;
    case "Paraxip.config.fxotune_run_calibration":
      next_item = "Paraxip.config.fxotune.front_form";
      break;

    default: // front panel form
      next_item = "Paraxip.config.fxotune.front_form";
      break;
    }
  }
  else if (direction == "back"){
    switch (active_id) {
    case "Paraxip.config.fxotune.select_lines":
      next_item = "Paraxip.config.fxotune.front_form";
      break;
    case "Paraxip.config.fxotune_run_calibration":
      next_item = "Paraxip.config.fxotune.select_lines";
      break;
    default: // front panel form
      next_item = "Paraxip.config.fxotune.front_form";
      break;
    }
  }

  // Determine element to show/hide based on step to become active
  switch (next_item) {
  case "Paraxip.config.fxotune.front_form":
    show_back = false;
    break;
  case "Paraxip.config.fxotune.select_lines":
    show_back = true;
    show_next = false;
    show_runCalibration = true;
    break;
  case "Paraxip.config.fxotune_run_calibration":
    show_back = false;
    show_next = false;
    show_runCalibration = false;
    show_cancelCalibration = true;
    break;
    
  default: 
    show_back = false;
    break;
  }

  lay.setActiveItem(next_item);
  lay.container.setTitle(lay.activeItem.title);
  
  fxoCalibration_next_button.setVisible(show_next);
  fxoCalibration_back_button.setVisible(show_back);
  fxoCalibration_runCalibration_button.setVisible(show_runCalibration);
  fxoCalibration_cancelCalibration_button.setVisible(show_cancelCalibration);
  fxoCalibration_progressBar.setVisible(show_cancelCalibration);
  fxoCalibration_finishCalibration_button.setVisible(false);
  
};

var calibrationStatusTask;
var cancelMode = false;

var fxoCalibration_back_button = new Ext.Button({
  id        : 'move-prev'
  ,text     : '<- Back'
  ,hidden   : true
  ,minWidth : 80
});

fxoCalibration_back_button.on('click', 
    fxoTuningNavHandler.createDelegate(fxoCalibration_back_button, ['back']));

var fxoCalibration_next_button = new Ext.Button({
  id        : 'move-next'
  ,text     : 'Next ->'
  ,minWidth : 80
});

fxoCalibration_next_button.on('click', 
    fxoTuningNavHandler.createDelegate(fxoCalibration_next_button, ['next']));

var fxoCalibration_runCalibration_button = new Ext.Button({
  id        : 'runCalibration'
  ,text     : 'Start Calibration'
  ,hidden   :true
  ,minWidth : 80
});
fxoCalibration_runCalibration_button.on('click', 
    fxoTuningNavHandler.createDelegate(fxoCalibration_next_button, ['next']));

var fxoCalibration_cancelCalibration_button = new Ext.Button({
  id        : 'cancelCalibration'
  ,text     : 'Cancel Calibration'
  ,hidden   : true
  ,minWidth : 80
});
fxoCalibration_cancelCalibration_button.on('click', 
    cancelCalibration.createDelegate(fxoCalibration_cancelCalibration_button));

var fxoCalibration_finishCalibration_button = new Ext.Button({
  id        : 'finishCalibration'
  ,text     : 'Finish'
  ,hidden   : true
  ,minWidth : 80
});
fxoCalibration_finishCalibration_button.on('click', 
    finishCalibration.createDelegate(fxoCalibration_finishCalibration_button));

var fxoCalibration_progressBar = new Ext.ProgressBar({
  id:'pbar3',
  width:300,
  hidden:true
});

var fxoTuneWizard = new Ext.FormPanel({
  title         : 'FXO Calibration Wizard'
  ,id           : 'fxoWizard'
  ,layout       : 'card'
  ,hideMode     : 'offsets'
  ,layoutConfig : { deferedRender: true, layoutOnTabChange: true }
  ,defaults     : { border: false }
  ,bbar         : [
    fxoCalibration_progressBar,     
    fxoCalibration_back_button,
    '->',
    fxoCalibration_next_button,
    fxoCalibration_cancelCalibration_button,
    fxoCalibration_runCalibration_button,
    fxoCalibration_finishCalibration_button,
    '',
    ''
  ]
});

var isRendered = false; //keep trace of the first render done by doLayout()

fxoTuneWizard.add(fxotune_front_form);
fxoTuneWizard.add(fxotune_select_lines_form);
fxoTuneWizard.add(fxotune_run_calibration);

fxoTuneWizard.on('afterlayout', function() {
  if (!isRendered) {
    fxoTuneWizard.setTitle(fxotune_front_form.title);
    isRendered = true;
  } else if (fxoTuneWizard.getLayout().activeItem === null) {
    fxoTuneWizard.getLayout().setActiveItem(fxotune_front_form);
    //fxoTuneWizard.getLayout().setActiveItem(fxotune_select_lines_form);
  }
});

Paraxip_config_fxotune_interfacesStore.addListener("load", function(){
  nbFxoInterfaces =
    Paraxip_config_fxotune_interfacesStore.reader.jsonData.data.length;

  var mainTab = Ext.getCmp('mainTabPanel');
  if(mainTab != undefined){
    var fxoTuningTab = mainTab.findById("fxoTuneTabId");
    fxoTuningTab.setDisabled(!(nbFxoInterfaces > 0));
  }
})
  
//update the fxo interface store
//enable/disable the tab (if it has or not fxo interfaces)
Paraxip.config.fxoTune.update = function(){
  fxoTuneWizard.doLayout();
  Paraxip_config_fxotune_interfacesStore.reload();
}



