Ext.namespace("Paraxip.status.channels");

// This map contains all the colors associated with the channel statuses.
Paraxip.status.channels.StatusColorMap = new Paraxip.Map();
Paraxip.status.channels.StatusColorMap.put('i','#ffff00');
Paraxip.status.channels.StatusColorMap.put('i*','#ffff00');
Paraxip.status.channels.StatusColorMap.put('a','#ff6347');
Paraxip.status.channels.StatusColorMap.put('a*','#ff6347');
Paraxip.status.channels.StatusColorMap.put('I','#87cefa');
Paraxip.status.channels.StatusColorMap.put('I*','#87cefa');
Paraxip.status.channels.StatusColorMap.put('A','#98fb98');
Paraxip.status.channels.StatusColorMap.put('A*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('D','#98fb98');
Paraxip.status.channels.StatusColorMap.put('D*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('R','#98fb98');
Paraxip.status.channels.StatusColorMap.put('R*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('C','#98fb98');
Paraxip.status.channels.StatusColorMap.put('C*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('H','#98fb98');
Paraxip.status.channels.StatusColorMap.put('H*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('G','#ffff00');
Paraxip.status.channels.StatusColorMap.put('G*','#ffff00');
Paraxip.status.channels.StatusColorMap.put('O','#ffff00');
Paraxip.status.channels.StatusColorMap.put('O*','#ffff00');
Paraxip.status.channels.StatusColorMap.put('','#ffffff');

// Variable to trace if it is the first call made to getChannelStatus or not.
Paraxip.status.channels.isFirstCall = true;

// Minimum width required for a proper display of the channels' grid display
Paraxip.status.channels.minGridWidth = 500;

var channelsStatusRefreshTime = 30;

Paraxip.status.channels.updateChannelGrid = function(){
  var gridView = Paraxip.status.channels.channelGrid.getView();
  var store    = Paraxip.status.channels.channelGrid.getStore();
  var colModel = Paraxip.status.channels.channelGrid.getColumnModel();
  var colCount = colModel.getColumnCount();
  var rowCount = store.getTotalCount();

  for( var i = 0; i < rowCount; i++ ) {
    for( var j = 1; j < colCount; j++ ) {
      var rec = store.getAt(i); // SelectionModel might not give you one single row!
      var colName = colModel.getDataIndex(j); // Columns might move!
      var val = rec.get(colName);
      var cell = gridView.getCell(i,j);
      var color = Paraxip.status.channels.StatusColorMap.get(val);
      cell.setAttribute('bgColor',color);
    }
  }
}

Paraxip.status.channels.quiesceChannels = function(){
  Ext.Ajax.request({
    url: Paraxip.baseUrl + "status/quiesceChannels",
    method: 'POST',
    disableCaching: true,
    params: {js:Ext.util.JSON.encode('dummy param')},
    success: function(){
      Paraxip.status.channels.updateChannelStatus();          
    },
    failure: Paraxip.failureMessage
  });
}

Paraxip.status.channels.channelsInService = function(){
  Ext.Ajax.request({
    url: Paraxip.baseUrl + "status/channelsInService",
    method: 'POST',
    disableCaching: true,
    params: {js:Ext.util.JSON.encode('dummy param')},
    success: function(){
      Paraxip.status.channels.updateChannelStatus();          
    },
    failure: Paraxip.failureMessage
  });
}

Paraxip.status.channels.updateChannelStatus = function(){
  Paraxip.status.channels.channelGrid.getStore().reload();
}

Paraxip.status.channels.resizeGrid = function(numCol,numRow){
  var width = 45 + (numCol-1) * 25;

  if( width < Paraxip.status.channels.minGridWidth ) {
    width = Paraxip.status.channels.minGridWidth;
  }
  
  var height = 103 + 20*(numRow);
  var x = (970-width) / 2;
  var y = (482-height) / 2;

  Paraxip.status.channels.channelGrid.setWidth(width);
  Paraxip.status.channels.channelGrid.setHeight(height);
  Paraxip.status.channels.channelGrid.setPosition(x,y);
  Paraxip.status.channels.channelGrid.ownerCt.doLayout();
}

Paraxip.status.channels.reconfigureGrid = function(data){
  colArray = new Array();
  fieldsArray = new Array();
  toolTipArray = new Array();
  var colHeaders = data.headers;
  var statusData = data.statusList;

  for( var i = 0; i < colHeaders.length; i++ ) {
    if ( i == 0) {
      var width = 40;
    } else {
      var width = 25;
    }
    colArray.push({header:colHeaders[i].headerName, width:width, dataIndex:colHeaders[i].dataIndex, css: 'text-align:center;'});
    fieldsArray.push({name:colHeaders[i].dataIndex});
  }

  // retriece the board name and interface name to display in tool tip
  for (var j = 0; j < statusData.length; j++ ) {
    toolTipArray.push({boardName:statusData[j].boardName,
                       interfaceName:statusData[j].interfaceName});
  }

  var store = new Ext.data.JsonStore({
    id        : 'channelGridStore',
    url       : Paraxip.baseUrl + "status/getChannelStatus",
    root      : 'data',
    fields    : fieldsArray,
    baseParams: { 
      firstCall:Ext.util.JSON.encode(Paraxip.status.channels.isFirstCall),
      reorderAnalog:true
    },
    listeners : {
      'load':Paraxip.status.channels.updateChannelGrid
    }
  });

  var colModel = new Ext.grid.ColumnModel(colArray);
  Paraxip.status.channels.channelGrid.reconfigure(store,colModel);
  Paraxip.status.channels.channelGrid.getStore().load({
    url:Paraxip.baseUrl + "status/getChannelStatus",
    callback: function(){
      var colCount = colModel.getColumnCount();
      var rowCount = store.getTotalCount();
      Paraxip.status.channels.resizeGrid(colCount,rowCount);
      Paraxip.status.channels.channelGrid.getStore().startAutoRefresh(
        Paraxip.status.channels.autoRefreshInterval.getValue());
    }
  });

}

Paraxip.status.channels.legendWindow = new Ext.Window({
  id          : 'Paraxip.status.channels.legendWindow',
  title       : 'Channel Status legend',
  closeAction : 'hide',
  plain       : true,
  hidden      : true,
  width       : 500,
  height      : 400,
  items       : [{
    xtype: 'label',
    html: "<html><br><font size='2'>I: Idle</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>i: initializing</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>a: alarmed</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>A: Answering</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>C: Connected</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>D: Dialing</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>R: Ringing</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>H: Hanging-up</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>G: Going-out-of-service</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>O: Out-of-service</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>F: out-of-service(Final)</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>*: Star appended to a status means the system is in transition to another status</font><br></html>"
  }]
});

Paraxip.status.channels.autoRefreshInterval = new Ext.form.NumberField({
  id: "Paraxip.status.channels.autoRefreshInterval"
  ,minValue: 1
  ,value: channelsStatusRefreshTime
  ,width: 30
  ,allowNegative: false
  ,allowDecimals: false
  ,allowBlank: false
  ,listeners: {
    'blur' : function() {
      var refreshRate = Paraxip.status.channels.autoRefreshInterval.getValue();
      if( refreshRate == 0 ) {
        Paraxip.status.channels.autoRefreshInterval.reset();
      }
      Paraxip.status.channels.channelGrid.getStore().stopAutoRefresh();
      Paraxip.status.channels.channelGrid.getStore().startAutoRefresh(
        Paraxip.status.channels.autoRefreshInterval.getValue());
    }
  }
});

Paraxip.status.channels.channelGrid = new Ext.grid.GridPanel({
  id: 'Paraxip.status.channels.channelGrid',
  title: 'PSTN Line Status',
  cls: 'channelGrid',
  autoScroll: true,
  stripeRows: true,
  enableHdMenu: false,
  enableColumnMove: false,
  hideHeaders: false,
  border: true,
  autoHeight: true,
  style: 'text-align:left;white-space: normal;',
  store: new Ext.data.JsonStore({
    id: 'channelGridStore',
    url: Paraxip.baseUrl + "status/getChannelStatus",
    baseParams: {
      firstCall:Ext.util.JSON.encode(Paraxip.status.channels.isFirstCall),
      reorderAnalog:true
    },
    fields: [
      {name: 'displayName'},
    ]
  }),
  columns: [],
  viewConfig :{forceFit: true, scrollOffset: 0, autoFill: true},
  buttonAlign : "center",
  bbar:[
  {
    xtype: "label"
    ,text: "Automatic Refresh Interval (s):"
  },
    Paraxip.status.channels.autoRefreshInterval,
    "-",
  {
    id: 'Paraxip.status.channels.refreshButton',
    text: 'Refresh',
    listeners: {
      'click':function(){
        Paraxip.status.channels.updateChannelStatus();
      }
    }
  },
    "-",
  {
    id: 'Paraxip.status.channels.showLegendButton',
    text: 'Show Legend',
    listeners: {
      'click':function(){
        Paraxip.status.channels.legendWindow.show();
      }
    }
  },
    "-",
  {
    id: 'Paraxip.status.channels.quiesceButton',
    text: 'Quiesce',
    listeners: {
      'click':Paraxip.status.channels.quiesceChannels
    }
  },
    "-",
  {
    id: 'Paraxip.status.channels.inServiceButton',
    text: 'In Service',
    listeners: {
      'click':Paraxip.status.channels.channelsInService
    }
  }]
});

Ext.override(Ext.ToolTip, {
  onTargetOver : function(e){
    if(this.disabled || e.within(this.target.dom, true)){
      return;
    }
    var t = e.getTarget(this.delegate);
    if (t){
      this.triggerElement = t;
      this.clearTimer('hide');
      this.targetXY = e.getXY();
      this.delayShow();
    }
  },
  onMouseMove : function(e){
    var t = e.getTarget(this.delegate);
    if (t) {
      this.targetXY = e.getXY();
      if (t === this.triggerElement) {
        if(!this.hidden && this.trackMouse){
          this.setPagePosition(this.getTargetXY());
        }
      } else {
        this.hide();
        this.lastActive = new Date(0);
        this.onTargetOver(e);
      }
    } else if (!this.closable && this.isVisible()) {
      this.hide();
    }
  },
  hide: function(){
    this.clearTimer('dismiss');
    this.lastActive = new Date();
    delete this.triggerElement;
    Ext.ToolTip.superclass.hide.call(this);
  }
});


Paraxip.status.channels.channelPanel = new Ext.Panel({
  id        : 'Paraxip.status.channels.channelPanel',
  layout    : 'absolute',
  width     : 971,
  height    : 486,
  items     : [Paraxip.status.channels.channelGrid],
  listeners : {
    'beforerender' : function(){
      if( Paraxip.gatewayStarted == undefined ) {
        Paraxip.isGwStartedCall();
      }
    }
  }
});

Paraxip.status.channels.rebuildGridRequest = function(){
  Paraxip.status.channels.isFirstCall = true;
  if( Paraxip.status.channels.channelGrid.rendered ) {
    Ext.Ajax.request({
      url: Paraxip.baseUrl + "status/getChannelStatus",
      method: 'GET',
      params: {
        firstCall:Ext.util.JSON.encode(Paraxip.status.channels.isFirstCall),
        reorderAnalog:true
      },
      success: function(form, action){
        Paraxip.status.channels.isFirstCall = false;
        var response = Ext.util.JSON.decode(form.responseText);
        Paraxip.status.channels.reconfigureGrid(response.data);
      }
    });
  }
}

Paraxip.status.channels.channelGrid.on('render',function(grid) {
  Ext.Ajax.request({
    url: Paraxip.baseUrl + "getGCParameterValue",
    disableCaching: true,
    params: {"paramName":'netborder.oamWebServer.channelsStatusRefreshTimeSec'},
    success: function(result){
      var response = Ext.util.JSON.decode(result.responseText);
      if (response.paramValue) {
        channelsStatusRefreshTime = response.paramValue;
        if( Paraxip.status.channels.autoRefreshInterval != undefined ) {
          Paraxip.status.channels.autoRefreshInterval.setValue(channelsStatusRefreshTime);
        }
      }
      Paraxip.status.channels.rebuildGridRequest();
      var tpl = new Ext.Template(
        '<b>{boardName}: {interfaceName}</b>');

      // tooltip code
      //var store = grid.getStore();  // Capture the Store.

      var view = grid.getView();    // Capture the GridView.

      Paraxip.status.channels.channelGrid.tip = new Ext.ToolTip({
        target: view.mainBody,    // The overall target element.

        delegate: '.x-grid3-row', // Each grid row causes its own seperate
                                // show and hide.

        trackMouse: true,         // Moving within the row should not hide the tip.

        renderTo: document.body,  // Render immediately so that tip.body can be
                                // referenced prior to the first show.

        width: 250,

        listeners: {  // Change content dynamically depending on which
                    // element triggered the show.

          beforeshow: function updateTipBody(tip) {
            //tip.body.dom.innerHTML = "Over Row " + view.findRowIndex(tip.triggerElement);
            var row = view.findRowIndex(tip.triggerElement);
            var rec = toolTipArray[row];
            tip.body.update(tpl.apply(rec));
          }
      
        }
      });
    }
  });
});

